#! /usr/bin/python

#/** 
 #* Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 #* 
 #* Based on code from ./cinnamon-multimonitor-switcher/cinnamon-multimonitor-switcher.py in xorg-server. 
 #* Copyright (c) 1997-2003 by The XFree86 Project, Inc.
 #* 
 #* This software is published by the license of CPU-OS License, you can use and 
 #* distibuted this software under this License. See CPU-OS Liense for more detail.
 #*
 #* You should have receive a copy of CPU-OS License. If not, please contact us 
 #* by email <support_os@cpu-os.ac.cn>  
 #* 
 #**/


import gettext
import os
import time
import dbus
import fcntl
import sys
import xml.dom.minidom
import xml.etree.ElementTree as xml2
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk, Gio
from xml.dom.minidom import parse

# gettext.install("cdos-desktop", "/usr/share/cdos-de/locale")
gettext.bindtextdomain("cdos-desktop", "/usr/share/cdos-de/locale")
gettext.textdomain("cdos-desktop")
_ = gettext.gettext

MIRRORED_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/mirror_1.png"
DOCKED_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/dock_1.png"
EXTENDED_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/extension_1.png"
INTERNAL_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/internal_1.png"

#CONFIG_FILE = "/usr/lib/cdos-multimonitor-switcher/mode.config"
#CONFIG_FILE = os.environ['HOME']+"/.config/mode.config"
RESOLUTION_FILE = os.environ['HOME'] + "/.config/monitors.xml"
F10_RESOLUTION_FILE = os.environ['HOME'] + "/.config/monitors_for_async.xml"
print RESOLUTION_FILE
#CONFIG_FILE = "/usr/lib/cdos-multimonitor-switcher/mode.config"
CONFIG_FILE = os.environ['HOME'] + "/.config/mode.config"
print CONFIG_FILE
GRAY_MIRRORED_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/gray_mirror.png"
GRAY_DOCKED_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/gray_dock.png"
GRAY_EXTENDED_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/gray_extension.png"
GRAY_INTERNAL_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/gray_internal.png"

SETTINGS = Gio.Settings.new('org.cdos')

def read_resolution(lvds, vga):
    #print lvds, vga
    try:
        xmldoc = xml.dom.minidom.parse(RESOLUTION_FILE)
        monitors = xmldoc.documentElement
    except Exception, e:
        print "Warning: cannot parse file: %s" %RESOLUTION_FILE
        return ("", "")
    lvds_resolution=""
    vga_resolution=""
    outputs = monitors.getElementsByTagName("output")
    for output in outputs:
        if output.childNodes.length <= 1:
            continue
        if output.hasAttribute("name"):
            #print "output name: %s" % output.getAttribute("name")
            name = output.getAttribute("name")
            if name == lvds:
                if output.hasAttribute("width"):
                    lvds_width = output.getElementsByTagName('width')[0]
                    lvds_height = output.getElementsByTagName('height')[0]
                    lvds_resolution = str(lvds_width.childNodes[0].data) + "x" + str(lvds_height.childNodes[0].data)
                else:
                    vga_resolution = str("1024") + "x" + str("768")
            if name == vga:
                if output.hasAttribute("width"):
                    vga_width = output.getElementsByTagName('width')[0]
                    vga_height = output.getElementsByTagName('height')[0]
                    vga_resolution = str(vga_width.childNodes[0].data) + "x" + str(vga_height.childNodes[0].data)
                else:
                    vga_resolution = str("1024") + "x" + str("728")

    if(lvds_resolution and vga_resolution):
        return (lvds_resolution, vga_resolution)
    else:
        return ("", "")

def get_primary(lvds, vga):
    #print lvds, vga
    try:
        xmldoc = xml.dom.minidom.parse(RESOLUTION_FILE)
        monitors = xmldoc.documentElement
    except Exception, e:
        print "Warning: cannot parse file: %s" %RESOLUTION_FILE
        return ""
    outputs = monitors.getElementsByTagName("output")
    for output in outputs:
        if output.childNodes.length <= 1:
            continue
        if output.hasAttribute("name"):
            #print "output name: %s" % output.getAttribute("name")
            name = output.getAttribute("name")
            if output.hasAttribute("primary"):
                primary_out = output.getElementsByTagName("primary")[0]
                res = str(primary_out.childNodes[0].data)
            else:
                return "lvds"
            #print res
            if res == "yes":
                return name
    return ""

def update_config(str):
    # print "str:%s" %str
    fcon = open(CONFIG_FILE, "w")
    try:
        fcon.write(str)
    finally:
        fcon.close()

def read_config():
    config_curr = ""
    fcon = open(CONFIG_FILE, "a+")
    try:
        config_curr = fcon.read().strip('\n')
    finally:
        fcon.close()

    return config_curr
    
def update_settings(mode):
    SETTINGS.set_int('multimonitor-switcher-mode', mode)
    
def get_settings_value():
    return SETTINGS.get_int('multimonitor-switcher-mode')

BLUE_MIRRORED_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/blue_mirror.png"
BLUE_DOCKED_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/blue_dock.png"
BLUE_EXTENDED_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/blue_extension.png"
BLUE_INTERNAL_ICON	= "/usr/lib/cdos-multimonitor-switcher/icons/blue_internal.png"

#MIRRORED_ICON	= "./icons/mirror.png"
#DOCKED_ICON	= "./icons/dock.png"
#EXTENDED_ICON	= "./icons/extension.png"
#INTERNAL_ICON	= "./icons/internal.png"

#GRAY_MIRRORED_ICON	= "./icons/gray_mirror.png"
#GRAY_DOCKED_ICON	= "./icons/gray_dock.png"
#GRAY_EXTENDED_ICON	= "./icons/gray_extension.png"
#GRAY_INTERNAL_ICON	= "./icons/gray_internal.png"

BLUE_MIRRORED_ICON	= "./icons/blue_mirror.png"
BLUE_DOCKED_ICON	= "./icons/blue_dock.png"
BLUE_EXTENDED_ICON	= "./icons/blue_extension.png"
BLUE_INTERNAL_ICON	= "./icons/blue_internal.png"



# Singleton
fh = 0
def run_once():
    global fh

    fh = open(os.path.realpath(__file__), 'r')

    try:
        fcntl.flock(fh, fcntl.LOCK_EX|fcntl.LOCK_NB)
    except:
        os._exit(0)
    finally:
        fh.close()

def get_connectors():
    vga_output = os.popen('xrandr | grep VGA | grep " connected " | awk \'{print $1}\'')
    vga = vga_output.read().strip('\n')
    crt_output = os.popen('xrandr | grep CRT | grep " connected " | awk \'{print $1}\'')
    crt = crt_output.read().strip('\n')
    if ((not vga.strip()) and crt):
        vga = crt

    lvds_output = os.popen('xrandr | grep LVDS | grep " connected " | awk \'{print $1}\'')
    lvds = lvds_output.read().strip('\n')
    dp_output = os.popen('xrandr | grep DP | grep " connected " | awk \'{print $1}\'')
    dp = dp_output.read().strip('\n')
    if ((not lvds.strip()) and dp):
        lvds = dp

    dp5_output = os.popen('xrandr | grep DP5 | grep " connected " | awk \'{print $1}\'')
    dp5 = dp5_output.read().strip('\n')
    dp6_output = os.popen('xrandr | grep DP6 | grep " connected " | awk \'{print $1}\'')
    dp6 = dp6_output.read().strip('\n')
    if (dp5 and dp6):
        lvds = dp5
        vga = dp6

    hdmi_output = os.popen('xrandr | grep HDMI | grep " connected " | awk \'{print $1}\'')
    hdmi = hdmi_output.read().strip('\n')
    if (hdmi.strip()):
        lvds = hdmi
        
    dis_output = os.popen('xrandr | grep DisplayPort | grep " connected " | awk \'{print $1}\'')
    dis = dis_output.read().strip('\n')
    if (dis.strip()):
        vga = dis
    
    return (vga, lvds)

def get_best_resolution():
    res = os.popen('/bin/bash /usr/lib/cdos-multimonitor-switcher/adjust.sh')
    r=res.read().split(' ')
    return r

class ProjectButton(Gtk.Button):
    def __init__(self, icon, label_name):
        super(ProjectButton, self).__init__()
        self.set_name('ProjectButton')
        self.set_relief(Gtk.ReliefStyle.NONE)
        self.set_size_request(156, 112)

        self.vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.add(self.vbox)

        self.image = Gtk.Image()
        self.image.set_from_file(icon)
        self.image.set_margin_bottom(3)

        self.label = Gtk.Label()
        self.label.set_use_markup(True)
        self.label.set_markup('<span foreground="#fff" font="14px">%s</span>' % label_name)
        self.label.set_margin_bottom(13)

        self.vbox.add(self.image)
        self.vbox.add(self.label)

    def set_selected(self, selected):
        if selected:
            self.set_state_flags(Gtk.StateFlags.SELECTED, True)
        else:
            self.unset_state_flags(Gtk.StateFlags.SELECTED)

class CMSWindow(Gtk.Window):
    def __init__(self):
        Gtk.Window.__init__(self, title=_("Cinnamon Multi-Monitor Switcher"))
        # Main window conf
        # self.set_border_width(10)
        self.set_position(Gtk.WindowPosition.CENTER)
        self.set_default_size(720, 260)
        self.set_resizable(False)
        self.set_decorated(False)
        self.set_icon_name("backgrounds")
        self.set_name('CMSWindow')
        self.load_css('/usr/lib/cdos-multimonitor-switcher/cdos-multimonitor-switcher.css')

        # dbus init
        #		bus = dbus.SessionBus()
        #		bus_obj = bus.get_object('org.gnome.SettingsDaemon', '/org/cdos/SettingsDaemon/XRANDR')
        #		self.iface = dbus.Interface(bus_obj, 'org.cdos.SettingsDaemon.XRANDR_2')

        # drawing GUI stuff
        self.vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.add(self.vbox)

        self.close_button = Gtk.Button()
        self.close_button.set_size_request(16, 16)
        self.close_button.set_property("width-request", 16)
        self.close_button.set_property("height-request", 16)
        self.close_button.set_name('close_button')
        self.close_button.set_relief(Gtk.ReliefStyle.NONE)
        self.close_button.connect("clicked", Gtk.main_quit)

        button_box = Gtk.ButtonBox.new(Gtk.Orientation.HORIZONTAL)
        button_box.set_layout(Gtk.ButtonBoxStyle.END)
        button_box.add(self.close_button)
        button_box.set_child_non_homogeneous(self.close_button, True)

        self.close_button_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        self.close_button_box.set_size_request(720, 30)
        self.close_button_box.set_resize_mode(Gtk.ResizeMode.QUEUE)
        self.close_button_box.pack_end(button_box, False, False, 0)

        self.title_label = Gtk.Label()
        self.title_label.set_use_markup(True)
        self.title_label.set_markup('<span foreground="#fff" font="18px">%s</span>' % _("Project"))

        self.title_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        self.title_box.set_size_request(-1, 18)
        self.title_box.set_halign(Gtk.Align.CENTER)
        self.title_box.add(self.title_label)
        self.title_box.set_margin_bottom(32)

        self.vbox.add(self.close_button_box)
        self.vbox.add(self.title_box)

        self.project_button_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        self.project_button_box.set_size_request(624, 112)
        self.project_button_box.set_margin_left(48)
        self.project_button_box.set_margin_right(48)
        self.project_button_box.set_margin_bottom(54)
        self.vbox.add(self.project_button_box)

        # Internal mode button
        self.i_button = ProjectButton(INTERNAL_ICON, _("Computer screen only"))
        self.i_button.connect("clicked", self.on_internal_button_clicked)
        self.project_button_box.pack_start(self.i_button, False, False, 0)

        # Mirrored mode button
        self.m_button = ProjectButton(MIRRORED_ICON, _("Duplicate"))
        self.m_button.connect("clicked", self.on_mirrored_button_clicked)
        self.project_button_box.pack_start(self.m_button, False, False, 0)

        # Docked monitor mode button
        self.d_button = ProjectButton(DOCKED_ICON, _("Second screen only"))
        self.d_button.connect("clicked", self.on_docked_button_clicked)
        self.project_button_box.pack_start(self.d_button, False, False, 0)

        # extended mode button
        self.e_button = ProjectButton(EXTENDED_ICON, _("Extend"))
        self.e_button.connect("clicked", self.on_extended_button_clicked)
        self.project_button_box.pack_start(self.e_button, False, False, 0)

        self.connect("key_press_event", self.key_press_event)

        self.mode = read_config()
        if self.mode == "":
            print "mode is null"
            self.mode = "4"
        # else:
        #     print mode
        if (self.mode == "1"):
            # self.set_focus(self.i_button)
            self.i_button.grab_focus()
            # childimage = self.i_button.get_child()
            # childimage.set_from_file(GRAY_INTERNAL_ICON)
            self.i_button.set_selected(True)
        elif (self.mode == "2"):
            # self.set_focus(self.m_button)
            self.m_button.grab_focus()
            # childimage = self.m_button.get_child()
            # childimage.set_from_file(GRAY_MIRRORED_ICON)
            self.m_button.set_selected(True)
        elif (self.mode == "3"):
            # self.set_focus(self.d_button)
            self.d_button.grab_focus()
            # childimage = self.d_button.get_child()
            # childimage.set_from_file(GRAY_DOCKED_ICON)
            self.d_button.set_selected(True)
        elif (self.mode == "4"):
            # self.set_focus(self.e_button)
            self.e_button.grab_focus()
            # childimage = self.e_button.get_child()
            # childimage.set_from_file(GRAY_EXTENDED_ICON)
            self.e_button.set_selected(True)

    def load_css(self, file_name):
        screen = Gdk.Screen.get_default()
        provider = Gtk.CssProvider()
        provider.load_from_path(file_name)
        context = Gtk.StyleContext()
        context.add_provider_for_screen(screen, provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

    def do_switch_internal(self):
        vga, lvds = get_connectors()
        #print("lvfei switch internal: test if file exist if exist,read it,if not use auto  ")
        resolution_lvds, resolution_vga = read_resolution(lvds, vga)
        #print resolution_lvds
        #print resolution_vga
        if (lvds and vga):
            #make all the audio to the internal analog audio sink
            os.system("/bin/bash /usr/lib/cdos-multimonitor-switcher/audio2analog.sh")
            #resolution_lvds, resolution_vga = read_resolution(lvds, vga)
            #os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
            is_exist = os.path.exists(RESOLUTION_FILE)
            if(is_exist and resolution_lvds):
                os.system("xrandr --output %s --off --output %s --mode %s" % (vga,lvds,resolution_lvds))
                #print("internal mode")
                update_config("1")
                update_settings(1)
            else:
                os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
                update_config("1")
                update_settings(1)
        else:
            #print("nothing... internal mode button pressed..")
            os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
            update_config("1")
            update_settings(1)

    def do_switch_mirror(self):
        vga, lvds = get_connectors()
        #print lvds, vga
        if (lvds and vga):
            res1, res2 = get_best_resolution()
            #print res1,res2
            #os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
            os.system("xrandr --output %s --mode %s --primary --output %s --mode %s --same-as %s" % (lvds, res1, vga, res1, lvds))
            #os.system("xrandr --output %s --mode %s --primary --output %s --mode %s --same-as %s" % (lvds, res1, vga, res2, lvds))
            #print ("mirrored mode")
            update_config("2")
            update_settings(2)
        else:
            print("nothing... mirrored mode button pressed..")

    def do_switch_docked(self):
        vga, lvds = get_connectors()
        resolution_lvds, resolution_vga = read_resolution(lvds, vga)
        #print ("in dock test the file is exist,if exist,read it ,if not use best ")
        #print resolution_lvds
        if (lvds and vga):
            is_exist = os.path.exists(RESOLUTION_FILE)
            if(is_exist and resolution_vga):
                os.system("xrandr --output %s --off --output %s --mode %s" % (lvds,vga,resolution_vga))
                #os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
                #os.system("xrandr --output %s --auto --output %s --off" % (vga, lvds))
                update_config("3")
                update_settings(3)
            else:
                #os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
                os.system("xrandr --output %s --auto --output %s --off" % (vga, lvds))
                #print ("update the  dock status")
                update_config("3")
                update_settings(3)
            #print("external mode")
            #update_config("1")
        else:
            print("nothing... docked mode button pressed..")
            #os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
            #os.system("xrandr --output %s --auto --output %s --off" % (vga, lvds))
            #update_config("3")
            #update_settings(3)

    def do_switch_extended(self):
        vga, lvds = get_connectors()
        primary_out = get_primary(lvds, vga)
        print ("lvfei is vga:%s,lvds:%s") %(vga,lvds)
        print "primary output is %s" %primary_out
        if (lvds and vga):
            resolution_lvds, resolution_vga = read_resolution(lvds, vga)
            os.system("xrandr --output %s --output %s" % (vga, lvds))
            if(resolution_lvds and resolution_vga ):
                if lvds == primary_out:
                    os.system("xrandr --output %s --mode %s --primary --output %s --mode %s --right-of %s" % (lvds, resolution_lvds, vga, resolution_vga, lvds))
                else:
                    os.system("xrandr --output %s --mode %s --primary --output %s --mode %s --right-of %s" % (vga, resolution_vga, lvds, resolution_lvds, vga))

            else:
                #os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
                os.system("xrandr --output %s --auto --primary --output %s --auto --right-of %s" % (lvds, vga, lvds))
            #print("extended mode.")
            update_config("4")
            update_settings(4)
        else:
            print("nothing... extended mode button pressed..")

    def write_to_config_xml(self,m, f, o, event):
        print event
        if event == Gio.FileMonitorEvent.CHANGES_DONE_HINT:
            print "okk"
        self.tree = xml2.ElementTree()
        self.tree.parse(F10_RESOLUTION_FILE)

        for elem in self.tree.iter():
            print elem.tag, elem.text
            if self.mode == 2:
            # mirror mode, clone yes
                if elem.tag == "clone":
                    elem.text = "yes"
        self.tree.write(RESOLUTION_FILE)
        Gtk.main_quit()

    def on_internal_button_clicked(self, widget):
        self.do_switch_internal()
        os.system("xset dpms force off")
        time.sleep(0.5)
        os.system("xset dpms force on")
        self.mode = 1

    def on_mirrored_button_clicked(self, widget):
        self.do_switch_mirror()
        os.system("xset dpms force off")
        time.sleep(0.5)
        os.system("xset dpms force on")
        self.mode = 2

    def on_docked_button_clicked(self, widget):
        self.do_switch_docked()
        os.system("xset dpms force off")
        time.sleep(0.5)
        os.system("xset dpms force on")
        self.mode = 3

    def on_extended_button_clicked(self, widget):
        self.do_switch_extended()
        os.system("xset dpms force off")
        time.sleep(0.5)
        os.system("xset dpms force on")
        self.mode = 4

    def key_press_event(self, widget, event):
        global timer_interval
        timer_interval=0
        keyname = Gdk.keyval_name(event.keyval)
        #print "Key %s (%d) was pressed" % (keyname, event.keyval)
        if(keyname == "Escape"):
            Gtk.main_quit()
        if((keyname =="p") or (keyname =="F10")):
            if(self.i_button==self.get_focus()):
                # print "i_button get_focus"
                # self.set_focus(self.m_button)
                self.m_button.grab_focus()
                # childimage = self.m_button.get_child()
                # childimage.set_from_file(GRAY_MIRRORED_ICON)
                # childimage = self.i_button.get_child()
                # childimage.set_from_file(INTERNAL_ICON)
                self.m_button.set_selected(True)
                self.i_button.set_selected(False)
            elif(self.m_button==self.get_focus()):
                # print "m_button get_focus"
                # self.set_focus(self.d_button)
                self.d_button.grab_focus()
                # childimage = self.d_button.get_child()
                # childimage.set_from_file(GRAY_DOCKED_ICON)
                # childimage = self.m_button.get_child()
                # childimage.set_from_file(MIRRORED_ICON)
                self.d_button.set_selected(True)
                self.m_button.set_selected(False)
            elif(self.d_button==self.get_focus()):
                # print "d_button get_focus"
                # self.set_focus(self.e_button)
                self.e_button.grab_focus()
                # childimage = self.e_button.get_child()
                # childimage.set_from_file(GRAY_EXTENDED_ICON)
                # childimage = self.d_button.get_child()
                # childimage.set_from_file(DOCKED_ICON)
                self.e_button.set_selected(True)
                self.d_button.set_selected(False)
            elif(self.e_button==self.get_focus()):
                # print "e_button get_focus"
                # self.set_focus(self.i_button)
                self.i_button.grab_focus()
                # childimage = self.i_button.get_child()
                # childimage.set_from_file(GRAY_INTERNAL_ICON)
                # childimage = self.e_button.get_child()
                # childimage.set_from_file(EXTENDED_ICON)
                self.i_button.set_selected(True)
                self.e_button.set_selected(False)
        elif(keyname=="Right"):
            if(self.i_button==self.get_focus()):
                # self.set_focus(self.m_button)
                # childimage = self.m_button.get_child()
                # childimage.set_from_file(GRAY_MIRRORED_ICON)
                # childimage = self.i_button.get_child()
                # childimage.set_from_file(INTERNAL_ICON)
                # self.m_button.grab_focus()
                self.m_button.set_selected(True)
                self.i_button.set_selected(False)
            elif(self.m_button==self.get_focus()):
                # self.set_focus(self.d_button)
                # childimage = self.d_button.get_child()
                # childimage.set_from_file(GRAY_DOCKED_ICON)
                # childimage = self.m_button.get_child()
                # childimage.set_from_file(MIRRORED_ICON)
                # self.d_button.grab_focus()
                self.d_button.set_selected(True)
                self.m_button.set_selected(False)
            elif(self.d_button==self.get_focus()):
                # self.set_focus(self.e_button)
                # childimage = self.e_button.get_child()
                # childimage.set_from_file(GRAY_EXTENDED_ICON)
                # childimage = self.d_button.get_child()
                # childimage.set_from_file(DOCKED_ICON)
                # self.e_button.grab_focus()
                self.e_button.set_selected(True)
                self.d_button.set_selected(False)
            elif(self.e_button==self.get_focus()):
                # self.set_focus(self.i_button)
                # childimage = self.e_button.get_child()
                # childimage.set_from_file(GRAY_EXTENDED_ICON)
                # childimage = self.i_button.get_child()
                # childimage.set_from_file(GRAY_INTERNAL_ICON)
                # self.i_button.grab_focus()
                # self.i_button.set_selected(True)
                # self.e_button.set_selected(False)
                self.e_button.set_selected(True)
        elif(keyname=="Left"):
            if(self.i_button==self.get_focus()):
                # self.set_focus(self.m_button)
                # childimage = self.m_button.get_child()
                # childimage.set_from_file(INTERNAL_ICON)
                # childimage = self.i_button.get_child()
                # childimage.set_from_file(GRAY_INTERNAL_ICON)
                # self.e_button.grab_focus()
                # self.e_button.set_selected(True)
                # self.i_button.set_selected(False)
                self.i_button.set_selected(True)
            elif(self.m_button==self.get_focus()):
                # self.set_focus(self.d_button)
                # childimage = self.i_button.get_child()
                # childimage.set_from_file(GRAY_INTERNAL_ICON)
                # childimage = self.m_button.get_child()
                # childimage.set_from_file(MIRRORED_ICON)
                # self.i_button.grab_focus()
                self.i_button.set_selected(True)
                self.m_button.set_selected(False)
            elif(self.d_button==self.get_focus()):
                # self.set_focus(self.e_button)
                # childimage = self.m_button.get_child()
                # childimage.set_from_file(GRAY_MIRRORED_ICON)
                # childimage = self.d_button.get_child()
                # childimage.set_from_file(DOCKED_ICON)
                # self.m_button.grab_focus()
                self.m_button.set_selected(True)
                self.d_button.set_selected(False)
            elif(self.e_button==self.get_focus()):
                # self.set_focus(self.i_button)
                # childimage = self.d_button.get_child()
                # childimage.set_from_file(GRAY_DOCKED_ICON)
                # childimage = self.e_button.get_child()
                # childimage.set_from_file(EXTENDED_ICON)
                # self.d_button.grab_focus()
                self.d_button.set_selected(True)
                self.e_button.set_selected(False)

run_once()

win = CMSWindow()
win.connect("delete-event", Gtk.main_quit)
win.connect("focus-out-event", Gtk.main_quit)

if( len(sys.argv) > 1):
    mode=sys.argv[1]
    print mode
    if (mode == "1"):
        win.do_switch_internal()
    elif (mode == "2"):
        win.do_switch_mirror()
    elif (mode == "3"):
        win.do_switch_docked()
    elif (mode == "4"):
        win.do_switch_extended()

else:
    print "ok"
    win.show_all()
    win.set_opacity(0.8)

    myfile = Gio.File.new_for_path(F10_RESOLUTION_FILE)
    monitor = myfile.monitor_file(Gio.FileMonitorFlags.NONE, None)
    monitor.connect("changed", win.write_to_config_xml)
    Gtk.main()

