#!/usr/bin/env python

#/** 
 #* Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 #* 
 #* Based on code from ./cinnamon-screensaver-lock-dialog/cinnamon-screensaver-lock-dialog.py in xorg-server. 
 #* Copyright (c) 1997-2003 by The XFree86 Project, Inc.
 #* 
 #* This software is published by the license of CPU-OS License, you can use and 
 #* distibuted this software under this License. See CPU-OS Liense for more detail.
 #*
 #* You should have receive a copy of CPU-OS License. If not, please contact us 
 #* by email <support_os@cpu-os.ac.cn>  
 #* 
 #**/


import sys
import os
import gettext
from gi.repository import Gtk
import pwd
import socket

# i18n
gettext.install("cdos-desktop", "/usr/share/cdos-de/locale")

class MainWindow:
  
    ''' Create the UI '''
    def __init__(self):

        user_id = os.getuid()
        username =  pwd.getpwuid(user_id).pw_name
        real_name = pwd.getpwuid(user_id).pw_gecos
        home_dir = pwd.getpwuid(user_id).pw_dir
        
        real_name = real_name.replace(",", "")
        if real_name == "":
            real_name = username        

        self.builder = Gtk.Builder()
        self.builder.add_from_file("/usr/lib/cdos-screensaver-lock-dialog/cdos-screensaver-lock-dialog.ui")
        
        self.window = self.builder.get_object("main_dialog")
        self.button_cancel = self.builder.get_object("button_cancel")
        self.button_ok = self.builder.get_object("button_ok")
        self.entry = self.builder.get_object("entry_away_message")
        self.image = self.builder.get_object("image_face")
                    
        self.window.set_title(_("Screen Locker"))
        self.window.set_icon_name("cs-screensaver")
                
        self.builder.get_object("label_description").set_markup("<i>%s</i>" % _("Please type an away message for the lock screen"))
        self.builder.get_object("label_away_message").set_markup("<b>%s: </b>" % real_name)
        
        if os.path.exists("%s/.face" % home_dir):
            self.image.set_from_file("%s/.face" % home_dir)
        else:
            self.image.set_from_file("/usr/share/pixmaps/nobody.png")
        
        self.window.connect("destroy", Gtk.main_quit)
        self.button_cancel.connect("clicked", Gtk.main_quit)
        self.button_ok.connect('clicked', self.lock_screen)
        self.entry.connect('changed', self.check_string)
        self.entry.connect('activate', self.lock_screen)
        
        self.builder.get_object("dialog-action_area1").set_focus_chain((self.button_ok, self.button_cancel))
          
        self.window.show()                    
        
    def lock_screen(self, data):
        message = self.entry.get_text()
        if (message != ""):
            os.system("cdos-screensaver-command --lock --away-message \'%s\' &" % self.entry.get_text())
        else:
            os.system("cdos-screensaver-command --lock &")
        Gtk.main_quit()
   
    def check_string(self, *args):
        message = self.entry.get_text()
        if len(message) >= 32:
            self.entry.set_text(message[0:31])
            self.entryask(_("please input text len less than 32"))
	      
    def entryask(self, msg):
        dialog = Gtk.MessageDialog(None,Gtk.DialogFlags.DESTROY_WITH_PARENT,Gtk.MessageType.ERROR,Gtk.ButtonsType.OK,None)
        dialog.set_default_size(400, 200)
        dialog.set_markup(msg)
        dialog.show_all()
        response = dialog.run()
        dialog.destroy()
        return response == Gtk.ResponseType.OK

if __name__ == "__main__":    
    MainWindow()
    Gtk.main()
