#!/usr/bin/env python

#/** 
 #* Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 #* 
 #* Based on code from ./cinnamon-settings/modules/cs_info.py in xorg-server. 
 #* Copyright (c) 1997-2003 by The XFree86 Project, Inc.
 #* 
 #* This software is published by the license of CPU-OS License, you can use and 
 #* distibuted this software under this License. See CPU-OS Liense for more detail.
 #*
 #* You should have receive a copy of CPU-OS License. If not, please contact us 
 #* by email <support_os@cpu-os.ac.cn>  
 #* 
 #**/


from SettingsWidgets import *

import platform
import subprocess
import os
import re
import glob
import threading
import shlex
import locale

# Add any other device pattern to read from
# dev_pattern = ['sd.*','mmcblk*', 'nvme0n1*']

def killProcess(process):
    process.kill()

def getProcessOut(command1, command2=None):
    timeout = 2.0  # Timeout for any subprocess before aborting it

    lines = []
    if command2 == None:
        p = subprocess.Popen(command1, stdout=subprocess.PIPE)
    else:
        p1 = subprocess.Popen(command1, stdout=subprocess.PIPE)
        p = subprocess.Popen(shlex.split(command2), stdin=p1.stdout, stdout=subprocess.PIPE)
    timer = threading.Timer(timeout, killProcess, [p])
    timer.start()
    while True:
        line = p.stdout.readline()
        if not line:
            break
        if line != '':
            lines.append(line)
    timer.cancel()
    return lines

def getGraphicsInfos():
    cards = {}
    count = 0
    for card in getProcessOut(("lspci")):
        if not "VGA" in card:
            continue
        cardId = card.split()[0]
        cardName = None
        for line in getProcessOut(("lspci", "-v", "-s", cardId)):
            if line.startswith(cardId):
                cardName = (line.split(":")[2].split("(rev")[0].strip())
  
        if cardName:
            cards[count] = (cardName)
            count += 1
    return cards
'''
def _size(device):
    nr_sectors = open(device+'/size').read().rstrip('\n')
    sect_size = open(device+'/queue/hw_sector_size').read().rstrip('\n')
    # The sect_size is in bytes, so we convert it to GiB and then send it back
    return (float(nr_sectors)*float(sect_size))/(1024.0*1024.0*1024.0)

def _getDiskSize():
    pattern = 'sda'
    root_part = os.popen("cat /etc/fstab | grep ^\"\# / \" | sed -n \"1,1p\" | awk -F ' ' '{print $5}'").read()
    
    if root_part:
        _root_part = root_part.strip()
        for tmp in dev_pattern:
            if re.compile(tmp).match(os.path.basename(_root_part)):
                pattern = tmp

    for device in glob.glob('/sys/block/*'):
        if re.compile(pattern).match(os.path.basename(device)):
            return _size(device)

def getDiskSize():
    size = 0
    size_str = ''    

    size = _getDiskSize()
    if size > 1024:
        size_str = str(round(size/1000, 2)) + " TB"
    else :
        size_str = str(round(size, 2)) + " GB"
    return size_str

    # temp=os.popen('udisksctl info -b /dev/sda | grep Size | awk -F\":\" \'{print $2}\'').read()
    # sizestr=float(temp.strip())
    # sizek = round(sizestr)
    # sizeT = sizek / 1024 ** 4 * 1.073741824
    # sizeT = 12 / 13
    # sizeG = sizek / 1024 **3
    # return sizeT
    # if sizeG > 900:
    #    return sizeT
    # else:
    #    return sizeG
'''

def getDiskSize():
    disksize = 0
    moreThanOnce = 0
    for line in getProcessOut(("df", "-l")):
        if line.startswith("/dev/"):
            moreThanOnce += 1
            disksize += float(line.split()[1])

    if (moreThanOnce > 1):
        return disksize, True
    else:
        return disksize, False

def getProcInfos():
    infos = [
        ("/proc/cpuinfo", [("cpu_name", "model name"), ("cpu_siblings", "siblings"), ("cpu_cores", "cpu cores")]),
        ("/proc/meminfo", [("mem_total", "MemTotal")])
    ]

    result = {}
    for (proc, pairs) in infos:
        for line in getProcessOut(("cat", proc)):
            for (key, start) in pairs:
                if line.startswith(start):
                    result[key] = line.split(':', 1)[1].strip()
                    break
    return result

def createSystemInfos():    
    procInfos = getProcInfos()
    infos = []
    (dname, dversion, dsuffix) = platform.linux_distribution()
    arch = platform.machine().replace("_", "-")    
    (memsize, memunit) = procInfos['mem_total'].split(" ")
    processorName = procInfos['cpu_name'].replace("(R)", u"\u00A9").replace("(TM)", u"\u2122")
    language = locale.getdefaultlocale()
    if 'cpu_cores' in procInfos:
        processorName = processorName + " x " + procInfos['cpu_cores']
    
    if os.path.exists("/etc/cdos/info"):
        language_first = language[0]
        if language_first == "zh_CN":
            title = commands.getoutput("awk -F \"=\" '/GRUB_TITLE_ZH/ {print $2}' /etc/cdos/info")
            infos.append((_("Operating System"), title))
        elif language_first == "en_US":
            title = commands.getoutput("awk -F \"=\" '/GRUB_TITLE_EN/ {print $2}' /etc/cdos/info")
            infos.append((_("Operating System"), title))
        else :
            title = commands.getoutput("awk -F \"=\" '/GRUB_TITLE/ {print $2}' /etc/cdos/info | sed -n \"1, 1p\"")
            infos.append((_("Operating System"), title))
    else:
        infos.append((_("Operating System"), dname + " " + dversion +  " '" + dsuffix.title() + "' (" + arch + ")"))    
#    if 'CINNAMON_VERSION' in os.environ:            
#        infos.append((_("CDOS Version"), os.environ['CINNAMON_VERSION']))
    infos.append((_("Linux Kernel"), platform.release()))
    if os.path.exists("/var/log/installer/syslog"):
        t=os.stat("/var/log/installer/syslog")
        infos.append((_('System Setup Time'),time.strftime('%Y-%m-%d  %H:%M',time.localtime(t.st_ctime))))
    infos.append((_("Processor"), processorName))
    if memunit == "kB":
        infos.append((_("Memory"), '%d %s' % ((round(float(memsize)/(1000*1000))), _("GB"))))
        #infos.append((_("Memory"), '%.2f %s' % ((float(memsize)/(1024*1024)), _("GB"))))
    else:
        infos.append((_("Memory"), procInfos['mem_total']))
    
    diskSize, multipleDisks = getDiskSize()
    if (multipleDisks):
        diskText = _("Hard Drives")
    else:
        diskText = _("Hard Drive")
    infos.append((diskText, '%.1f %s' % ((diskSize / (1000*1000)), _("GB"))))

    #diskSize = getDiskSize()
    #if diskSize != '':
    #    infos.append((_("Hard Drive"), (diskSize)))
    # print("%d\n",diskSize)
    # if diskSize <= 10:
    #     infos.append((_("Hard Drive"), '%d %s' % ((round(diskSize)),_("TB"))))
    # else:
    #     infos.append((_("Hard Drive"), '%d %s' % ((round(diskSize)),_("GB"))))

    cards = getGraphicsInfos()
    for card in cards:
        infos.append((_("Graphics Card"), cards[card]))

    return infos

class Module:
    def __init__(self, content_box):
        keywords = _("system, information, details, graphic, sound, kernel, version")
        sidePage = SidePage(_("System Info"), "cs-details1", keywords, content_box, module=self)
        self.sidePage = sidePage
        self.name = "info"
        self.category = "hardware"
        self.comment = _("Display system information")
        
    def on_module_selected(self):
        if not self.loaded:
            print "Loading Info module"
            infos = createSystemInfos()                        
            
            table = Gtk.Table.new(len(infos), 2, False)
            table.set_row_spacings(8)
            table.set_col_spacings(15)
            self.sidePage.add_widget(table)

            row = 0
            for (key, value) in infos:
                labelKey = Gtk.Label.new(key)
                labelKey.set_alignment(1, 0.5)
                labelKey.get_style_context().add_class("dim-label")
                labelValue = Gtk.Label.new(value)
                labelValue.set_alignment(0, 0.5)
                table.attach(labelKey, 0, 1, row, row+1)
                table.attach(labelValue, 1, 2, row, row+1)
                row += 1
                
