#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#/** 
 #* Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 #* 
 #* Based on code from ./cinnamon-settings/modules/cs_notifications.py in xorg-server. 
 #* Copyright (c) 1997-2003 by The XFree86 Project, Inc.
 #* 
 #* This software is published by the license of CPU-OS License, you can use and 
 #* distibuted this software under this License. See CPU-OS Liense for more detail.
 #*
 #* You should have receive a copy of CPU-OS License. If not, please contact us 
 #* by email <support_os@cpu-os.ac.cn>  
 #* 
 #**/

import os

# use our local typelib
os.environ['GI_TYPELIB_PATH'] = 'libcdosnotify:' + os.environ.get('GI_TYPELIB_PATH', '')

from SettingsWidgets import *
import gi
gi.require_version('CdosNotify', '1.0')
gi.require_version('Gtk', '3.0')
from gi.repository import Gio, Gtk, GLib, CdosNotify
import subprocess
import sys   #test

settings = Gio.Settings.new('org.cdos')

def get_icon_path(name, size):
    iconTheme = Gtk.IconTheme.get_default()
    try:
        iconInfo = iconTheme.lookup_icon(name, size, 0)
        path = iconInfo.get_filename()
    except AttributeError:
        path = '/usr/share/icons/CDOS-Classic/status/32/image-missing.png'

    return path

class AppButton(Gtk.Button):
    def __init__(self, app_name, icon, size, widget_name):
        print "AppButton:", sys._getframe().f_code.co_name  # 获取当前函数名
        super(AppButton, self).__init__()
        self.set_relief(Gtk.ReliefStyle.NONE)
        self.set_size_request(200, 48)
        #用于设置CSS样式
        self.set_name(widget_name)

        #self.connect('clicked', self.on_button_clicked)

        self.buttonHbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        self.add(self.buttonHbox)

        # image = Gtk.Image.new_from_icon_name(icon_name, icon_size)
        fileName = get_icon_path(icon, size)
        print "fileName= ", fileName  # 调试
        self.image = Gtk.Image.new_from_file(fileName)
        self.image.set_valign(Gtk.Align.CENTER)

        self.labelVbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 2)
        self.labelVbox.set_margin_left(20)
        # labelName = Gtk.Label.new(app_name)
        self.labelName = Gtk.Label()
        self.labelName.set_use_markup(True)
        self.labelName.set_markup('<span foreground="#4c4c4c" font="14px">%s</span>' % app_name)

        self.labelState = Gtk.Label()
        self.labelState.set_use_markup(True)

        self.labelName.set_halign(Gtk.Align.START)
        self.labelState.set_halign(Gtk.Align.START)
        self.labelVbox.pack_start(self.labelName, False, False, 0)
        self.labelVbox.pack_start(self.labelState, False, False, 0)
        self.labelVbox.set_valign(Gtk.Align.CENTER)

        self.buttonHbox.pack_start(self.image, False, False, 0)
        self.buttonHbox.pack_start(self.labelVbox, False, False, 0)

        self.settings = settings
        self.settings.connect('changed::notification-all-app-switch', self.on_all_app_switch_changed)

        #应用按钮初始状态
        self.set_init_state()

    def set_state(self, str):
        self.labelState.set_markup('<span foreground="#a6a6a6" font="14px">%s</span>' % str)

    def get_name(self):
        app_name = self.labelName.get_text()

        return app_name

    def on_all_app_switch_changed(self, settings, key):
        print sys._getframe().f_code.co_name  # 获取当前函数名
        if settings.get_boolean('notification-all-app-switch'):
            self.set_sensitive(True)
        else:
            self.set_sensitive(False)

    def set_init_state(self):
        print "\n"
        print "SettingSwitch:", sys._getframe().f_code.co_name  # 获取当前函数名
        print "app_name= ", self.get_name()

        if not self.settings.get_boolean('notification-all-app-switch'):
            self.set_sensitive(False)

        if self.get_name() == _("System Update"):
            value = 'notification-system-update'
            name = "cdos-update"
        # elif self.get_name() == _("Music Player"):
        #     value = 'notification-music-player'
        #     name = _("Rhythmbox")
        elif self.get_name() == _("Printer"):
            value = 'notification-printer'
            name = _("Printers")
        elif self.get_name() == _("Browser"):
            value = 'notification-browser'
            name = _("cdos-browser2")

        notification, banner, sound = self.settings.get_value(value)
        # print "notification= %d, banner= %d, sound= %d" %(notification, banner, sound)
        print "notification: ", notification
        print "banner: ", banner
        print "sound: ", sound

        if notification == True:
            if banner == True and sound == True:
                print "in clear"
                self.set_state(_("Open: panel, banner, sound"))
                # CdosNotify.add_notify_blacklist_clear(_(name))
                CdosNotify.add_notify_blacklist_clear(name)
            elif banner == True and sound == False:
                print "in silent"
                self.set_state(_("Open: panel, banner"))
                # CdosNotify.add_notify_blacklist_silent(_(name))
                CdosNotify.add_notify_blacklist_silent(name)
            elif banner == False and sound == True:
                print "in dnd"
                self.set_state(_("Open: panel, sound"))
                # CdosNotify.add_notify_blacklist_dnd(_(name))
                CdosNotify.add_notify_blacklist_dnd(name)
            else:
                # (true, false, false)  ?
                print "in clean"
                self.set_state(_("Open: panel"))
                # CdosNotify.add_notify_blacklist_clean(_(name))
                CdosNotify.add_notify_blacklist_clean(name)
        else:
            self.set_state(_("Close"))
            # CdosNotify.add_notify_blacklist_clean(name)
            CdosNotify.add_notify_blacklist_block(name)

class SettingSwitch(Gtk.VBox):
    def __init__(self, widget_name, page_name):
        print "SettingSwitch:", sys._getframe().f_code.co_name  # 获取当前函数名
        super(SettingSwitch, self).__init__()
        self.switch = Gtk.Switch.new()
        self.switch.set_name(widget_name)

        self.pack_start(self.switch, False, False, 0)
        self.set_valign(Gtk.Align.CENTER)

        self.settings = settings
        self.settings.connect('changed::notification-all-app-switch', self.all_app_switch_changed)

        self.set_init_state(widget_name, page_name)

    def set_active(self, active):
        self.switch.set_active(active)

    def get_active(self):
        return self.switch.get_active()

    def set_sensitive(self, sensitive):
        self.switch.set_sensitive(sensitive)

    def connect(self, signal, callback):
        self.switch.connect(signal, callback)

    def all_app_switch_changed(self, settings, key):
        print sys._getframe().f_code.co_name  # 获取当前函数名
        widget_name = self.switch.get_name()
        if widget_name == 'system_update_switch' or \
           widget_name == 'mail_switch' or \
           widget_name == 'music_player_switch' or \
           widget_name == 'printer_switch' or \
           widget_name == 'browser_switch':
            if self.settings.get_boolean('notification-all-app-switch'):
                self.set_sensitive(True)
            else:
                self.set_sensitive(False)

    def set_init_state(self, widget_name, page_name):
        print '\n'
        print "SettingSwitch:", sys._getframe().f_code.co_name  # 获取当前函数名
        if widget_name == 'system_update_switch' or page_name == _("System Update"):
            value = 'notification-system-update'
        elif widget_name == 'music_player_switch' or page_name == _("Music Player"):
            value = 'notification-music-player'
        elif widget_name == 'printer_switch' or page_name == _("Printer"):
            value = 'notification-printer'
        elif widget_name == 'browser_switch' or page_name == _("Browser"):
            value = 'notification-browser'
        else:
            pass

        # 通知中心主页面上的开关
        if page_name == _("Notification Center"):
            notification = self.settings.get_value(value)[0]
            # print "notification= %d, banner= %d, sound= %d" %(notification, banner, sound)
            print "notification: ", notification
            # print "banner: ", banner
            # print "sound: ", sound

            if notification == True:
                self.set_active(True)
            else:
                self.set_active(False)

            if not self.settings.get_boolean('notification-all-app-switch'):
                self.set_sensitive(False)

        # 通知中心子页面上的开关
        if page_name == _("System Update") or \
           page_name == _("Music Player") or \
           page_name == _("Printer") or \
           page_name == _("Browser"):
            notification, banner, sound = self.settings.get_value(value)
            if widget_name == "notification_switch":
                if notification == True:
                    self.set_active(True)
                else:
                    self.set_active(False)

            if widget_name == "banner_switch":
                if banner == True:
                    self.set_active(True)
                else:
                    self.set_active(False)

            if widget_name == "sound_switch":
                if sound == True:
                    self.set_active(True)
                else:
                    self.set_active(False)


'''
class ButtonBrowser(Gtk.HBox):
    def __init__(self):
        super(ButtonBrowser, self).__init__()
        self.button = Gtk.Button.new()
        self.button.set_size_request(193, 32)
        self.button.set_name("button_browser")
        self.button.set_halign(Gtk.Align.START)
        self.button.set_margin_top(7)
        self.label = Gtk.Label()
        self.label.set_use_markup(True)
        name = _("Go to browser for more settings")
        self.label.set_markup('<span foreground="#fff" font="14px">%s</span>' % name)
        self.button.add(self.label)

        self.pack_start(self.button, False, False, 0)
'''

class DetailedSettingsPage(SidePage):
    def __init__(self, name, content_box):
        super(DetailedSettingsPage, self).__init__(name, "", "", content_box = content_box)
        self.detailSetVbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)

        self.grid = Gtk.Grid.new()
        self.grid.set_column_spacing(450)
        self.grid.set_row_spacing(25)
        self.grid.set_margin_left(30)
        self.grid.set_margin_top(20)
        self.detailSetVbox.pack_start(self.grid, False, False, 0)
        # self.detailSetVbox.pack_start(buttonBrowser, False, False, 0)

        self.notificationLabel = self.create_setting_label(_("Notification (Don't show in panel when close)"))
        self.notificationSwitch = SettingSwitch("notification_switch", name)
        self.grid.attach(self.notificationLabel, 0, 0, 1, 1)
        self.grid.attach(self.notificationSwitch, 1, 0, 1, 1)

        self.bannerLabel = self.create_setting_label(_("Show notification banner"))
        self.bannerSwitch = SettingSwitch("banner_switch", name)
        self.grid.attach(self.bannerLabel, 0, 1, 1, 1)
        self.grid.attach(self.bannerSwitch, 1, 1, 1, 1)

        self.soundLabel = self.create_setting_label(_("Sound reminder notification arrival"))
        self.soundSwitch = SettingSwitch("sound_switch", name)
        self.grid.attach(self.soundLabel, 0, 2, 1, 1)
        self.grid.attach(self.soundSwitch, 1, 2, 1, 1)

        self.buttonBrowser = self.create_button_browser()
        # self.buttonBrowser = ButtonBrowser()
        self.grid.attach(self.buttonBrowser, 0, 3, 1, 1)
        # self.detailSetVbox.pack_start(self.buttonBrowser, False, False, 0)

        self.add_widget(self.detailSetVbox)
        self.build()

        self.buttonBrowser.connect('clicked', self.go_to_browser)

        self.button_backward.set_sensitive(True)

        self.settings = settings
        if name == _("System Update"):
            #若初始值为(false, false, false)，置灰详细设置页面的横幅开关、声音开关
            self.update_switches(self.settings, 'notification-system-update')
            self.settings.connect('changed::notification-system-update', self.update_switches)
        if name == _("Music Player"):
            self.update_switches(self.settings, 'notification-music-player')
            self.settings.connect('changed::notification-music-player', self.update_switches)
        if name == _("Printer"):
            self.update_switches(self.settings, 'notification-printer')
            self.settings.connect('changed::notification-printer', self.update_switches)
        if name == _("Browser"):
            self.update_switches(self.settings, 'notification-browser')
            self.settings.connect('changed::notification-browser', self.update_switches)

    def create_setting_label(self, name):
        labelVbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)

        #label = Gtk.Label.new(name)
        label = Gtk.Label()
        label.set_use_markup(True)
        label.set_markup('<span foreground="#4c4c4c" font="14px">%s</span>' % name)

        labelVbox.pack_start(label, False, False, 0)
        labelVbox.set_halign(Gtk.Align.START)
        labelVbox.set_valign(Gtk.Align.CENTER)

        return labelVbox

    def create_button_browser(self):
        buttonBrowser = Gtk.Button.new()
        buttonBrowser.set_size_request(193, 32)
        buttonBrowser.set_name("button_browser")
        buttonBrowser.set_halign(Gtk.Align.START)
        buttonBrowser.set_margin_top(7)
        labelBrowser = Gtk.Label()
        labelBrowser.set_use_markup(True)
        name = _("Go to browser for more settings")
        labelBrowser.set_markup('<span foreground="#fff" font="14px">%s</span>' % name)
        buttonBrowser.add(labelBrowser)

        return buttonBrowser

    #详细设置页面更新开关状态
    def update_switches(self, settings, key):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        print "key= ", key

        #TODO 可能不需要判断key
        # if key == 'notification-music-player':
        notification, banner, sound = settings.get_value(key)
        print "notification=%s, banner=%s, sound=%s" % (notification, banner, sound)

        if notification == True:
            #若主页的通知开关打开，详细设置页面的通知开关随之打开
            if self.notificationSwitch.get_active() == False:
                self.notificationSwitch.set_active(True)

            self.bannerSwitch.set_sensitive(True)
            self.soundSwitch.set_sensitive(True)

            self.bannerSwitch.set_active(banner)
            self.soundSwitch.set_active(sound)
        else:
            if self.notificationSwitch.get_active() == True:
                self.notificationSwitch.set_active(False)

            #通知开关关闭时，关闭横幅开关，关闭声音开关，并将横幅开关和声音开关置灰
            if self.bannerSwitch.get_active() == True:
                self.bannerSwitch.set_active(False)

            if self.soundSwitch.get_active() == True:
                self.soundSwitch.set_active(False)

            self.bannerSwitch.set_sensitive(False)
            self.soundSwitch.set_sensitive(False)

    def go_to_browser(self, button):
        print sys._getframe().f_code.co_name  # 获取当前函数名
        subprocess.Popen('cdos-browser2 --no-sandbox', stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)

class Module:
    def __init__(self, content_box, main_window):
        keywords = _("notification, banner, sound")
        sidePage = SidePage(_("Notification Center"), "cs-notifications1", keywords, content_box, module=self, main_window=main_window)
        self.sidePage = sidePage
        self.comment = _("Set up notification reception,banner display and sound alert")
        self.name = "notifications"
        self.category = "prefs"
        self.content_box = content_box
        self.sidePage1 = None
        self.switchAllApp = None
        self.update_button = None
        self.update_switch = None
        # self.mail_button = None
        # self.mail_switch = None
        # self.player_button = None
        # self.player_switch = None
        self.printer_button = None
        self.browser_button = None
        self.browser_switch = None
        self.buttonBrowser = None
        self.current_setting_page = None
        self.last_setting_page = None
        self.settings = settings
        self.main_window = main_window
        self.sidePages = []
        self.button_backward = self.sidePage.button_backward
        self.button_forward = self.sidePage.button_forward

        self.set_theme('/usr/lib/cdos-settings/notification.css')

    def set_theme(self, file_theme):
        screen = Gdk.Screen.get_default()
        provider = Gtk.CssProvider()
        provider.load_from_path(file_theme)
        context = Gtk.StyleContext()
        context.add_provider_for_screen(screen, provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

    def on_active(self, switch, data):
        print sys._getframe().f_code.co_name  # 获取当前函数名
        if switch.get_active():
            settings.set_boolean('notification-all-app-switch', True)
            CdosNotify.add_notify_blacklist_unblock_all()
        else:
            settings.set_boolean('notification-all-app-switch', False)
            CdosNotify.add_notify_blacklist_block_all()

    def all_application_switch(self):
        #labelAllApp = Gtk.Label.new(_("All application notification switch"))
        labelAllApp = Gtk.Label()
        name = _("All application notification switch")
        labelAllApp.set_use_markup(True)
        labelAllApp.set_markup('<span foreground="#4c4c4c" font="16px">%s</span>' % name)

        #labelDescription1 = Gtk.Label.new(_("Toggle this switch to set up global notifications"))
        labelDescription1 = Gtk.Label()
        name = _("Toggle this switch to set up global notifications")
        labelDescription1.set_use_markup(True)
        labelDescription1.set_markup('<span foreground="#a6a6a6" font="12px">%s</span>' % name)

        #labelDescription2 = Gtk.Label.new(_("Click on the following app to open detailed settings, set individually for each item"))
        labelDescription2 = Gtk.Label()
        name = _("Click on the following app to open detailed settings, set individually for each item")
        labelDescription2.set_use_markup(True)
        labelDescription2.set_markup('<span foreground="#a6a6a6" font="12px">%s</span>' % name)

        labelAllApp.set_halign(Gtk.Align.START)
        labelDescription1.set_halign(Gtk.Align.START)
        labelDescription2.set_halign(Gtk.Align.START)

        labelVbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 4)
        labelVbox.set_margin_left(30)
        labelVbox.pack_start(labelAllApp, False, False, 0)
        labelVbox.pack_start(labelDescription1, False, False, 0)
        labelVbox.pack_start(labelDescription2, False, False, 0)

        switchVbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.switchAllApp = Gtk.Switch.new()
        #self.switchAllApp.set_margin_right(90)  #?调整字体样式后没用
        self.switchAllApp.connect('notify::active', self.on_active)
        active = settings.get_boolean('notification-all-app-switch')
        self.switchAllApp.set_active(active)
        switchVbox.pack_start(self.switchAllApp, False, False, 0)
        switchVbox.set_margin_right(90)  #?调整字体样式后没用

        hbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        hbox.pack_start(labelVbox, False, False, 0)
        hbox.pack_end(switchVbox, False, False, 0)

        self.allAppVbox.pack_start(hbox, False, False, 0)

    def create_setting_label(self, name):
        labelVbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)

        #label = Gtk.Label.new(name)
        label = Gtk.Label()
        label.set_use_markup(True)
        label.set_markup('<span foreground="#4c4c4c" font="12px">%s</span>' % name)

        labelVbox.pack_start(label, False, False, 0)
        labelVbox.set_halign(Gtk.Align.START)
        labelVbox.set_valign(Gtk.Align.CENTER)

        return labelVbox

    def on_switch_changed(self, switch, active):
        print sys._getframe().f_code.co_name  # 获取当前函数名
        print "switch_name= ", switch.get_name()  # 调试

        if switch.get_name() == 'system_update_switch':
            value = 'notification-system-update'
            app_name = "cdos-update"
            button = self.update_button
        # elif switch.get_name() == 'music_player_switch':
        #     value = 'notification-music-player'
        #     app_name = 'Rhythmbox'
        #     button = self.player_button
        elif switch.get_name() == 'printer_switch':
            value = 'notification-printer'
            app_name = _("Printers")
            button = self.printer_button
        elif switch.get_name() == 'browser_switch':
            value = 'notification-browser'
            app_name = _("cdos-browser2")
            button = self.browser_button
        else:
            pass

        notification, banner, sound = self.settings.get_value(value)

        if switch.get_active():
            notification = True

            if banner == True and sound == True:
                print "in clear"
                button.set_state(_("Open: panel, banner, sound"))
                CdosNotify.add_notify_blacklist_clear(app_name)
            elif banner == True and sound == False:
                print "in silent"
                button.set_state(_("Open: panel, banner"))
                CdosNotify.add_notify_blacklist_silent(app_name)
            elif banner == False and sound == True:
                print "in dnd"
                button.set_state(_("Open: panel, sound"))
                CdosNotify.add_notify_blacklist_dnd(app_name)
            elif banner == False and sound == False:
                # (true, false, false) ?
                print "in clean"
                button.set_state(_("Open: panel"))
                CdosNotify.add_notify_blacklist_clean(app_name)
            else:
                pass
        else:
            notification = False
            button.set_state(_("Close"))
            # CdosNotify.add_notify_blacklist_clean(app_name)
            CdosNotify.add_notify_blacklist_block(app_name)

        self.settings.set_value(value, GLib.Variant('(bbb)', (notification, banner, sound)))

    def create_button_browser(self):
        buttonBrowser = Gtk.Button.new()
        buttonBrowser.set_size_request(193, 32)
        buttonBrowser.set_name("button_browser")
        buttonBrowser.set_halign(Gtk.Align.START)
        buttonBrowser.set_margin_top(7)
        labelBrowser = Gtk.Label()
        labelBrowser.set_use_markup(True)
        name = _("Go to browser for more settings")
        labelBrowser.set_markup('<span foreground="#fff" font="14px">%s</span>' % name)
        buttonBrowser.add(labelBrowser)

        return buttonBrowser

    def get_current_setting_page_state(self):
        if self.current_setting_page.name == _("System Update"):
            value = self.settings.get_value('notification-system-update')
        elif self.current_setting_page.name == _("Mail"):
            value = self.settings.get_value('notification-mail')
        elif self.current_setting_page.name == _("Music Player"):
            value = self.settings.get_value('notification-music-player')
        elif self.current_setting_page.name == _("Printer"):
            value = self.settings.get_value('notification-printer')
        elif self.current_setting_page.name == _("Browser"):
            value = self.settings.get_value('notification-browser')
        else:
            pass

        return value

    def get_notify_app_name(self):
        if self.current_setting_page.name == _("System Update"):
            app_name = "cdos-update"
        # elif self.current_setting_page.name == _("Music Player"):
        #     app_name = _("Rhythmbox")
        elif self.current_setting_page.name == _("Printer"):
            app_name = _("Printers")
        elif self.current_setting_page.name == _("Browser"):
            app_name = _("cdos-browser2")
        else:
            pass

        return app_name

    def notify_clear(self):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名

        CdosNotify.add_notify_blacklist_clear(self.get_notify_app_name())

    def notify_silent(self):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        CdosNotify.add_notify_blacklist_silent(self.get_notify_app_name())

    def notify_dnd(self):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        CdosNotify.add_notify_blacklist_dnd(self.get_notify_app_name())

    def notify_clean(self):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        CdosNotify.add_notify_blacklist_clean(self.get_notify_app_name())

    def notify_block(self):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        CdosNotify.add_notify_blacklist_block(self.get_notify_app_name())

    def update_setting_value(self, notification, banner, sound):
        if self.current_setting_page.name == _("System Update"):
            key = 'notification-system-update'
        elif self.current_setting_page.name == _("Music Player"):
            key = 'notification-music-player'
        elif self.current_setting_page.name == _("Printer"):
            key = 'notification-printer'
        elif self.current_setting_page.name == _("Browser"):
            key = 'notification-browser'
        else:
            pass

        self.settings.set_value(key, GLib.Variant('(bbb)', (notification, banner, sound)))

    def notification_switch_active(self, switch, active):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        print "current page name= ", self.current_setting_page.name

        notification, banner, sound = self.get_current_setting_page_state()
        print "notification= %s, banner= %s, sound= %s" % (notification, banner, sound)

        if switch.get_active():
            notification = True
            if banner == True and sound == True:
                self.notify_clear()
            elif banner == True and sound == False:
                self.notify_silent()
            elif banner == False and sound == True:
                self.notify_dnd()
            elif banner == False and sound == False:
                self.notify_clean()
            else:
                pass
        else:
            notification = False
            # self.notify_clean()
            self.notify_block()

        self.update_setting_value(notification, banner, sound)

    def banner_switch_active(self, switch, active):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        print "current page name= ", self.current_setting_page.name

        notification, banner, sound = self.get_current_setting_page_state()
        print "notification= %s, banner= %s, sound= %s" % (notification, banner, sound)
        if notification == False:
            return

        if switch.get_active():
            banner = True
            if sound == True:
                self.notify_clear()
            else:
                self.notify_silent()
        else:
            banner = False
            if sound == True:
                self.notify_dnd()
            else:
                self.notify_clean()

                # notification = False

        self.update_setting_value(notification, banner, sound)

    def sound_switch_active(self, switch, active):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        print "current page name= ", self.current_setting_page.name

        notification, banner, sound = self.get_current_setting_page_state()
        if notification == False:
            return

        if switch.get_active():
            sound = True
            if banner == True:
                self.notify_clear()
            else:
                self.notify_dnd()
        else:
            sound = False
            if banner == True:
                self.notify_silent()
            else:
                self.notify_clean()

                # notification = False

        self.update_setting_value(notification, banner, sound)

    def go_to_detailed_settings_page(self, button):
        # 如果子页面已存在，直接显示
        if len(self.sidePages) > 0:
            print "\n"
            print "previous page"
            for sidePage in self.sidePages:
                # print "button name: ", button.get_name()
                print "page name: ", sidePage.name
                if button.get_name() == sidePage.name:
                    print "in page show"
                    sidePage.build()
                    self.main_window.window.set_title(sidePage.name)
                    sidePage.buttonBrowser.hide()
                    if button.get_name() == _("Browser"):
                        print "in buttonBrowser show"
                        sidePage.buttonBrowser.show()

                    self.main_window.backward_button_show(sidePage.button_backward)
                    self.main_window.forward_button_show(sidePage.button_forward)

                    self.current_setting_page = sidePage
                    self.last_setting_page = self.current_setting_page

                    return

        self.main_window.window.set_title(button.get_name())
        self.sidePage1 = DetailedSettingsPage(button.get_name(), self.content_box)
        #“转到浏览器进行更多设置”按钮必须在调用过SidePage的build()方法后才能被隐藏，否则无法隐藏
        self.sidePage1.buttonBrowser.hide()
        if button.get_name() == _("Browser"):
            print "in buttonBrowser show"
            self.sidePage1.buttonBrowser.show()

        self.main_window.backward_button_show(self.sidePage1.button_backward)
        self.main_window.forward_button_show(self.sidePage1.button_forward)
        self.sidePage1.button_backward.connect('clicked', self.go_to_notification_center_page)

        self.current_setting_page = self.sidePage1
        self.last_setting_page = self.current_setting_page
        self.button_forward.set_sensitive(True)
        self.sidePages.append(self.sidePage1)

        self.sidePage1.notificationSwitch.connect('notify::active', self.notification_switch_active)
        self.sidePage1.bannerSwitch.connect('notify::active', self.banner_switch_active)
        self.sidePage1.soundSwitch.connect('notify::active', self.sound_switch_active)

    def go_to_notification_center_page(self, button):
        print sys._getframe().f_code.co_name  # 获取当前函数名
        self.sidePage.build()
        self.main_window.window.set_title(_("Notification Center"))
        self.button_forward.set_sensitive(True)
        self.main_window.backward_button_show(self.button_backward)
        self.main_window.forward_button_show(self.button_forward)

    def on_button_clicked(self, button):
        print sys._getframe().f_code.co_name  # 获取当前函数名
        print "button_name= ", button.get_name()   #调试

        self.go_to_detailed_settings_page(button)

    def app_settings(self):
        grid = Gtk.Grid.new()
        grid.set_column_spacing(386)
        grid.set_row_spacing(4)
        grid.set_margin_left(50)
        grid.set_margin_right(90)
        grid.set_margin_top(35)
        self.allAppVbox.pack_start(grid, False, False, 0)

        self.update_button = AppButton(_("System Update"), "system-upgrade", 32, "system_update")
        self.update_switch = SettingSwitch("system_update_switch", _("Notification Center"))
        grid.attach(self.update_button, 0, 0, 1, 1)
        grid.attach(self.update_switch, 1, 0, 1, 1)

        #去掉邮件和音乐播放器设置
        '''
        self.mail_button = AppButton(_("Mail"), "youtube", 32, "mail")
        self.mail_switch = SettingSwitch("mail_switch", _("Notification Center"))
        grid.attach(self.mail_button, 0, 1, 1, 1)
        grid.attach(self.mail_switch, 1, 1, 1, 1)

        self.player_button = AppButton(_("Music Player"), "rhythmbox", 32, "music_player")
        self.player_switch = SettingSwitch("music_player_switch", _("Notification Center"))
        grid.attach(self.player_button, 0, 2, 1, 1)
        grid.attach(self.player_switch, 1, 2, 1, 1)

        self.printer_button = AppButton(_("Printer"), "printer", 32, "printer")
        self.printer_switch = SettingSwitch("printer_switch", _("Notification Center"))
        grid.attach(self.printer_button, 0, 3, 1, 1)
        grid.attach(self.printer_switch, 1, 3, 1, 1)

        self.browser_button = AppButton(_("Browser"), "nfs-browser", 32, "browser")
        self.browser_switch = SettingSwitch("browser_switch", _("Notification Center"))
        grid.attach(self.browser_button, 0, 4, 1, 1)
        grid.attach(self.browser_switch, 1, 4, 1, 1)
        '''

        self.printer_button = AppButton(_("Printer"), "printer", 32, "printer")
        self.printer_switch = SettingSwitch("printer_switch", _("Notification Center"))
        grid.attach(self.printer_button, 0, 1, 1, 1)
        grid.attach(self.printer_switch, 1, 1, 1, 1)

        self.browser_button = AppButton(_("Browser"), "cdos-browser2", 32, "browser")
        self.browser_switch = SettingSwitch("browser_switch", _("Notification Center"))
        grid.attach(self.browser_button, 0, 2, 1, 1)
        grid.attach(self.browser_switch, 1, 2, 1, 1)

        self.update_button.connect('clicked', self.on_button_clicked)
        # self.mail_button.connect('clicked', self.on_button_clicked)
        # self.player_button.connect('clicked', self.on_button_clicked)
        self.printer_button.connect('clicked', self.on_button_clicked)
        self.browser_button.connect('clicked', self.on_button_clicked)

        self.update_switch.connect('notify::active', self.on_switch_changed)
        # self.mail_switch.connect('notify::active', self.on_switch_changed)
        # self.player_switch.connect('notify::active', self.on_switch_changed)
        self.printer_switch.connect('notify::active', self.on_switch_changed)
        self.browser_switch.connect('notify::active', self.on_switch_changed)

    def settings_changed(self, settings, key):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名

        if key == 'notification-system-update':
            notification, banner, sound = settings.get_value(key)
            if notification == True:
                self.update_switch.set_active(True)

                if banner == True and sound == True:
                    print "in clear"
                    self.update_button.set_state(_("Open: panel, banner, sound"))
                elif banner == True and sound == False:
                    print "in silent"
                    self.update_button.set_state(_("Open: panel, banner"))
                elif banner == False and sound == True:
                    print "in dnd"
                    self.update_button.set_state(_("Open: panel, sound"))
                else:
                    #(true, false, false)  ?
                    print "in clean"
                    self.update_button.set_state(_("Open: panel"))
            else:
                self.update_switch.set_active(False)

                self.update_button.set_state(_("Close"))

        '''
        if key == 'notification-music-player':
            notification, banner, sound = settings.get_value(key)
            if notification == True:
                self.player_switch.set_active(True)

                if banner == True and sound == True:
                    print "in clear"
                    self.player_button.set_state(_("Open: panel, banner, sound"))
                elif banner == True and sound == False:
                    print "in silent"
                    self.player_button.set_state(_("Open: panel, banner"))
                elif banner == False and sound == True:
                    print "in dnd"
                    self.player_button.set_state(_("Open: panel, sound"))
                else:
                    #(true, false, false)  ?
                    print "in clean"
                    self.player_button.set_state(_("Open: panel"))
            else:
                self.player_switch.set_active(False)

                self.player_button.set_state(_("Close"))
        '''

        if key == 'notification-printer':
            notification, banner, sound = settings.get_value(key)
            if notification == True:
                self.printer_switch.set_active(True)

                if banner == True and sound == True:
                    print "in clear"
                    self.printer_button.set_state(_("Open: panel, banner, sound"))
                elif banner == True and sound == False:
                    print "in silent"
                    self.printer_button.set_state(_("Open: panel, banner"))
                elif banner == False and sound == True:
                    print "in dnd"
                    self.printer_button.set_state(_("Open: panel, sound"))
                else:
                    #(true, false, false)  ?
                    print "in clean"
                    self.printer_button.set_state(_("Open: panel"))
            else:
                self.printer_switch.set_active(False)

                self.printer_button.set_state(_("Close"))

        if key == 'notification-browser':
            notification, banner, sound = settings.get_value(key)
            if notification == True:
                self.browser_switch.set_active(True)

                if banner == True and sound == True:
                    print "in clear"
                    self.browser_button.set_state(_("Open: panel, banner, sound"))
                elif banner == True and sound == False:
                    print "in silent"
                    self.browser_button.set_state(_("Open: panel, banner"))
                elif banner == False and sound == True:
                    print "in dnd"
                    self.browser_button.set_state(_("Open: panel, sound"))
                else:
                    #(true, false, false)  ?
                    print "in clean"
                    self.browser_button.set_state(_("Open: panel"))
            else:
                self.browser_switch.set_active(False)

                self.browser_button.set_state(_("Close"))

    def go_to_system_settings_page(self, button):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        self.last_setting_page = None
        self.button_forward.set_sensitive(False)
        self.main_window.button_back.clicked()

    def go_to_last_setting_page(self, button):
        print "\n"
        print sys._getframe().f_code.co_name  # 获取当前函数名
        self.main_window.window.set_title(self.last_setting_page.name)
        self.last_setting_page.build()
        self.last_setting_page.buttonBrowser.hide()
        if self.last_setting_page.name == _("Browser"):
            self.last_setting_page.buttonBrowser.show()
        self.main_window.backward_button_show(self.last_setting_page.button_backward)
        self.main_window.forward_button_show(self.last_setting_page.button_forward)

    def on_module_selected(self):
        if not self.loaded:
            print "Loading Notifications module"
            self.allAppVbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)

            self.all_application_switch()
            self.app_settings()

            self.settings.connect('changed::notification-system-update', self.settings_changed)
            self.settings.connect('changed::notification-music-player', self.settings_changed)
            self.settings.connect('changed::notification-printer', self.settings_changed)
            self.settings.connect('changed::notification-browser', self.settings_changed)
            
            self.sidePage.add_widget (self.allAppVbox)

            self.button_backward.set_sensitive(True)
            self.main_window.backward_button_show(self.button_backward)
            self.main_window.forward_button_show(self.button_forward)

            self.button_backward.connect('clicked', self.go_to_system_settings_page)
            self.button_forward.connect('clicked', self.go_to_last_setting_page)
